#include "IOTCClient.h"
#include "RDTClient.h"
#include "common.h"
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#define RDT_USERNAME "admin"
#define RDT_PASSWORD "testpwd"

void RDT_Client_Authentication(char *cszAccount, uint32_t nAccountMaxLength, char *cszPassword, uint32_t nPasswordMaxLength, const void *pArg) {
    printf("[%s] nAccountMaxLength=%d, nPasswordMaxLength=%d, arg=%s\n", __func__, nAccountMaxLength, nPasswordMaxLength, (char *)pArg);
    strcpy(cszAccount, RDT_USERNAME);
    strcpy(cszPassword, RDT_PASSWORD);
}

int main(int argc, char *argv[]) {
    int ret = 0;
    const char *license_key = NULL;
    IOTCConnectInput auth_option = {0};
    int use_key = 0;

    if (argc < 3) {
        printf("No UID or File Number!!!\n");
        printf("RDTClient [UID] [FileNum]\n");
        printf("RDTClient [UID] [FileNum] [AuthKey]\n");
        return 0;
    }

    auth_option.cb = sizeof(IOTCConnectInput);
    auth_option.authentication_type = AUTHENTICATE_BY_KEY;
    memset(auth_option.auth_key, '0', 8);
    auth_option.timeout = 0;

    if (argc > 3) {
        if (strlen(argv[3]) == 8)
        {
            use_key = 1;

            strncpy(auth_option.auth_key, argv[3], 8);
            printf("Connect with Key=%s\n", argv[3]);
        } else {
            printf("Invalid Key=%s, Connect without Key\n", argv[3]);
        }
    } else {
        printf("Connect without Key\n");
    }

    printf("RDT Version[%s]\n", RDT_GetRDTApiVersionString());
    int fileNum = atoi(argv[2]);

    //===== set license key for enable TUTK SDK =====
    license_key = getenv(ENV_LICENSE_KEY);
    if (license_key == NULL) {
        printf("Environment variable %s must be set!!\n", ENV_LICENSE_KEY);
        return -1;
    }

    //Set log file path
    LogAttr log_attr;
    memset(&log_attr, 0, sizeof(log_attr));

    log_attr.path = "./log_iotc_client.txt";
    ret = IOTC_Set_Log_Attr(log_attr);
    printf("IOTC_Set_Log_Attr ret[%d]\n", ret);

    log_attr.path = "./log_rdt_client.txt";
    ret = RDT_Set_Log_Attr(log_attr);
    printf("RDT_Set_Log_Attr ret[%d]\n", ret);

    log_attr.path = "./log_global_client.txt";
    ret = TUTK_Set_Log_Attr(log_attr);
    printf("TUTK_Set_Log_Attr ret[%d]\n", ret);

    ret = TUTK_SDK_Set_License_Key(license_key);
    printf("TUTK_SDK_Set_License_Key() ret[%d]\n", ret);
    if (ret != TUTK_ER_NoERROR) {
        return 0;
    }

    ret = IOTC_Initialize2(0);
    if (ret != IOTC_ER_NoERROR) {
        printf("IOTC_Initialize error!!\n");
        return 0;
    }

    int rdtCh = RDT_Initialize();
    if (rdtCh <= 0) {
        printf("RDT_Initialize error!!\n");
        return 0;
    }

    int SID, i = 0;
    char *UID = (char *)argv[1];

    while (1) {
        SID = IOTC_Get_SessionID();
        if (SID == IOTC_ER_NOT_INITIALIZED) {
            printf("Not Initialize!!!\n");
            return 0;
        } else if (SID == IOTC_ER_EXCEED_MAX_SESSION) {
            printf("EXCEED MAX SESSION!!!\n");
            return 0;
        }

        if(use_key)
            ret = IOTC_Connect_ByUIDEx(UID, SID, &auth_option);
        else
            ret = IOTC_Connect_ByUID_Parallel(UID, SID);
        printf("Step 2: call IOTC_Connect_ByUID_Parallel(%s) ret = %d\n", UID, ret);
        if (ret < 0) {
            printf("p2pAPIs_Client connect failed...!!\n");
            return 0;
        } else if (ret > -1)
            break;
    }

    char *args = strdup("test_args");
    int nRDTIndex = RDT_Client_Create(SID, 0, RDT_Client_Authentication, args, 3000);
    if (nRDTIndex < 0) {
        printf("RDT_Create failed[%d]!!\n", nRDTIndex);
        IOTC_Session_Close(SID);
        exit(0);
    }
    printf("RDT_Create OK[%d]\n", nRDTIndex);
    char buf[MAX_BUF_SIZE];
    sprintf(buf, "%d", fileNum);
    ret = RDT_Write(nRDTIndex, buf, strlen(buf));
    if (ret < 0) {
        printf("RDT send cmd failed[%d]!!\n", ret);
        IOTC_Session_Close(SID);
        exit(0);
    }

    ret = RDT_Read(nRDTIndex, buf, 1024, RDT_WAIT_TIMEMS);
    if (ret < 0) {
        printf("RDT rcv file size failed[%d]!!\n", ret);
        IOTC_Session_Close(SID);
        exit(0);
    }
    int fileSize = atoi(buf), remainReadSize;

    strcpy(buf, START_STRING);
    ret = RDT_Write(nRDTIndex, buf, strlen(buf));
    if (ret < 0) {
        printf("RDT send start failed[%d]!!\n", ret);
        IOTC_Session_Close(SID);
        exit(0);
    }

    printf("fileSize[%d]\n", fileSize);
    // FILE *fp;
    char fn[32];
    struct st_RDT_Status rdtStatus;
    for (i = 0; i < fileNum; i++) {
        if (RDT_Status_Check(nRDTIndex, &rdtStatus) < 0) {
            printf("RDT_Status_Check error!\n");
            break;
        }
        remainReadSize = fileSize;
        sprintf(fn, "%d.jpg", i);
#if 0
		FILE *fp = fopen(fn, "w+");
		if(fp == NULL)
		{
			printf("Open file[%s] error!!\n", fn);
			break;
		}
#endif
        while (1) {
            if (remainReadSize < MAX_BUF_SIZE)
                ret = RDT_Read(nRDTIndex, buf, remainReadSize, RDT_WAIT_TIMEMS);
            else
                ret = RDT_Read(nRDTIndex, buf, MAX_BUF_SIZE, RDT_WAIT_TIMEMS);

            if (ret < 0 && ret != RDT_ER_TIMEOUT) {
                printf("RDT_Read data failed[%d]!!", ret);
                break;
            }

            if (ret > 0) {
                printf("RDT_Read[%d]\n", ret);
                remainReadSize -= ret;
            }
            // fwrite(buf, 1, ret, fp);
            // fflush(fp);
            if (remainReadSize == 0)
                break;
        }
        // fclose(fp);
        printf("rcv file OK [%d]\n", i);
    }

    // printf("RDT_Destroy calling....\n");
    RDT_Destroy(nRDTIndex);
    printf("RDT_Destroy calling....OK!\n");
    IOTC_Session_Close(SID);
    printf("IOTC_Session_Close OK!\n");
    RDT_DeInitialize();
    printf("RDT_DeInitialize OK!\n");
    IOTC_DeInitialize();
    printf("*****clien exit*****\n");

    return 0;
}
